##########################################################################
###
### Copyright (C) 2007 Jason D. Nielsen
###
### This program is free software# you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation# either version 2 of the License, or
### (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY# without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program# if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA
###
##########################################################################

library(splines)

system("make")
system("make clean")
dyn.load(paste(getwd(),"/fm_NHPP.so",sep=""))

load("cb_dat.Robj")

fmNHPP<-function(Data,X,Panel,dfgmm,nboot=5000,tgrp=nrow(Data),sgsp=FALSE) {
###  R wrapper for Fortran code for finding spline NHPP coef
###       Data :  nxp matrix of counts
###       X    :  nxntr design matrix
###       Panel:  vector (px1) of Panel times
###       dfgmm:  degrees of freedom for penalty function
###       nboot:  number of bootsrap resamples to take, should be > 1000
###       sgsp :  Fit a constant penalty, (default is spline penalty)
###       tgrp :  vector, number of individuals in each treatment group
###               for bootstrap blocking.
###
###
###  Returns:  list containing est. coefs, est. functions, VarCov's and other useful
###            summary information
  
  n<-nrow(Data)
  p<-ncol(Data)
  ntr<-ncol(X)
  if (nrow(X)!=n) {
    stop("fmNHPP: dimensions of design matrix incorrectly specified#")
  }
  if (length(Panel)!=p) {
    stop("fmNHPP: ### of panels and panel counts don't match#")
  }
  if (dfgmm < 2) {
    stop("fmNHPP: if df of gmm is less then 2 just use a constant penalty, i.e. set sgsp=TRUE")
  }
  if (p > 30) {
    nkn<-30
  }
  else {
    nkn<-p
  }
  ncf<-ntr+nkn+2
### Number of knots in penalty spline  
  nst<-nkn-1
###  
  Dat<-t(Data)
  cfspl<-rep(0,ncf+nst+5)
  VCov<-rep(0,(ncf+1)*(ncf+1))
  twl<-runif(nboot*n)
  twl<-matrix(twl,nboot,n)
  twl[,1:tgrp[1]]<-tgrp[1]*twl[,1:tgrp[1]]
  if (length(tgrp) > 1) {
    for (i in 2:length(tgrp)) {
      twl[,(tgrp[i-1]+1):(tgrp[i-1]+tgrp[i])]<-tgrp[i]*twl[,(tgrp[i-1]+1):(tgrp[i-1]+tgrp[i])]+tgrp[i-1]
    }
  }  
  twl<-ceiling(twl)
  bcoef<-rep(0,nboot*(ncf+1))
  cat("Fitting Algorithm: ")
  FortRtn<-.Fortran("r_fmnhpp",as.double(Panel),as.double(Dat),as.double(X),as.double(dfgmm),as.integer(twl),
                    bcoef=as.double(bcoef),coef=as.double(cfspl),VCov=as.double(VCov),pql=as.double(0),
                    edf=as.double(0),as.integer(p),as.integer(n),as.integer(ntr),as.integer(nkn),
                    as.integer(nst),as.integer(nboot),as.logical(sgsp),err=as.integer(0))
  if (FortRtn$err != 0 ) {
    cat("[Error] \n")
    return(0)
  }
  cat("[Done] \n")
  coefs<-FortRtn$coef
  bcoef<-matrix(FortRtn$bcoef,ncf+1,nboot)
  bCIl<-apply(bcoef,1,quantile,prob=0.025)
  bCIu<-apply(bcoef,1,quantile,prob=0.975)
  bSE<-apply(bcoef,1,sd)
  edf<-FortRtn$edf
  pql<-FortRtn$pql
  jcoef<-coefs[1:(ncf+nst+3)]
  VrCov<-matrix(FortRtn$VCov,ncf+1,ncf+1)
  VrCov[abs(VrCov)<1e-7]<-0
  SE<-sqrt(diag(VrCov))
  tm<-1:200/200
  B<-trbasis(tm,nkn)
  Te<-Panel[p]
  tm<-tm*Te
  lam<-c(exp(B%*%coefs[c(ntr:ncf)]))
  Lam<-cumsum(tm[1]*lam)
  VV<-VrCov[ntr:ncf,ntr:ncf]
  lmvr<-B%*%VV%*%t(B)
  lmvr<-sqrt(diag(lmvr))
  lwlm<-lam*exp(-1.9599*lmvr)
  uplm<-lam*exp(1.9599*lmvr)
  lwLm<-cumsum(tm[1]*lwlm)
  upLm<-cumsum(tm[1]*uplm)
  lam<-cbind(lam,lwlm,uplm)
  Lam<-cbind(Lam,lwLm,upLm)
  blam<-exp(B%*%bcoef[ntr:ncf,])
  bLam<-tm[1]*blam
  bLam<-apply(bLam,2,cumsum)
  uplm<-apply(blam,1,quantile,prob=0.975)
  lwlm<-apply(blam,1,quantile,prob=0.025)
  upLm<-apply(bLam,1,quantile,prob=0.975)
  lwLm<-apply(bLam,1,quantile,prob=0.025)
  lam<-cbind(lam,lwlm,uplm)
  Lam<-cbind(Lam,lwLm,upLm)  
  if (ntr > 1) {
    rwnames<-paste("Beta",1:(ntr-1),sep="")
  }
  else {
    rwnames<-NULL
  }  
  rwnames<-c(rwnames,paste("Psi",1:(nkn+3),sep=""))
  rwnames<-c(rwnames,"Tau")  
  rwnames<-c(rwnames,paste("Spar",1:(nst+2),sep=""))
  rwnames<-c(rwnames,"SprUni")
  rwnames<-c(rwnames,"Gamma")  
  clnames<-c("Est","SE","bSE","bCIl","bCIu")
  coefs<-cbind(coefs,c(SE,rep(NA,nst+4)),c(bSE,rep(NA,nst+4)),c(bCIl,rep(NA,nst+4)),c(bCIu,rep(NA,nst+4)))
  colnames(coefs)<-clnames
  out<-NULL
  rownames(coefs)<-rwnames
  out$coef<-coefs
  out$VarCov<-VrCov
  out$edf<-edf
  out$pql<-pql
  out$lam<-lam
  out$Lam<-Lam
  out$Panel<-Panel
  out$Te<-Te
  out$n<-n
  out$p<-p
  out$nknt<-nkn
  out$nstp<-nst
  out$ncf<-ncf
  out$ntr<-ntr
  out
}

plot.fmNHPP<-function(obj) {
  vV<-obj$VarCov
  coefs<-obj$coef
  coefs<-coefs[,1]
  lam<-obj$lam
  Lam<-obj$Lam
  Te<-obj$Te
  ncf<-obj$ncf
  nstp<-obj$nstp
  tm<-1:200/200
  tm<-tm*Te
  par(mfrow=c(3,1))
  par(bg="white")
  matplot(tm,lam[,1:3],type="l",ylab=expression(paste(lambda[o],"(t)")),xlab="t",col=c(1,2,2),ylim=c(0,max(lam[,1:3])))
  matplot(tm,Lam[,1:3],type="l",ylab=expression(paste(Lambda[o],"(t)")),xlab="t",col=c(1,2,2))  
  spru<-coefs[ncf+nstp+4]
  spar<-coefs[(ncf+2):(ncf+nstp+3)]
  tm<-1:200/200
  B<-trbasis1(tm,nstp)
  gamm<-B%*%spar
  tm<-tm*Te
  matplot(tm,gamm,type="l",ylab=expression(paste(delta,"(t)")),xlab="t",col=1,lty=1)
  lines(c(0,Te),c(spru,spru),lwd=2)
}

summary.fmNHPP<-function(obj) {
  print(round(obj$coef,3))
  cat("\n")
}

trbasis<-function(x,nk) {
###      t       in: sx1 vector of times
###      nk      in: number of knots (integer)
###
### Description:
###      Computes the truncated power spline basis
###
  degree<-2
  knots<-(1:nk)/(nk+1)
  B<-rep(x,max(degree,nk))
  B<-matrix(B,ncol=max(degree,nk))
  trunP<-t(t(B[,1:degree])^(1:degree))
  trunP<-cbind(1,trunP)
  B<-B[,1:nk]
  B<-t(t(B)-c(knots))
  B[B<0]<-0
  B<-B^degree
  trunP<-cbind(trunP,B)
  trunP
}

trbasis1<-function(x,nk) {
###      t       in: sx1 vector of times
###      nk      in: number of knots (integer)
###
### Description:
###      Computes the truncated power spline basis
###
  degree<-1
  knots<-(1:nk)/(nk+1)
  B<-rep(x,max(degree,nk))
  B<-matrix(B,ncol=max(degree,nk))
  trunP<-t(t(B[,1:degree])^(1:degree))
  trunP<-cbind(1,trunP)
  B<-B[,1:nk]
  B<-t(t(B)-c(knots))
  B[B<0]<-0
  B<-B^degree
  trunP<-cbind(trunP,B)
  trunP
}

